package edu.georgetown.gucs.matcher;


import edu.georgetown.gucs.fingerprinter.Fingerprint;
import edu.georgetown.gucs.utility.ObjectBuilder;
import edu.georgetown.gucs.utility.Pair;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;


/**
 * @author by Evan B.
 * @since 1/20/15.
 */
public class BulkMatcherWorker implements Callable<List<Pair<String, Integer>>> {

    private int start;
    private FingerprintMatcher matcher;
    private List<Fingerprint> fingerprints;
    private boolean update;
    public BulkMatcherWorker(Integer start,
                                String matcher,
                                List<Fingerprint> fingerprints) {

        this.matcher = (FingerprintMatcher) ObjectBuilder.buildObject(matcher);
        this.start = start;
        this.fingerprints = fingerprints;

        update = false;
    }

    public void updateOutput() {

        this.update = true;

    }

    @Override
    public List<Pair<String, Integer>> call() throws Exception {

        ArrayList<Pair<String, Integer>> results = new ArrayList<>();


        for (int i = start; i < fingerprints.size(); i++) {

            int currentScore;
            //initial fingerprint
            Fingerprint compare = fingerprints.get(start);
            //fingerprints that will be compared to the initial fingerprint
            Fingerprint compareTo = fingerprints.get(i);

            if (matcher.toString().equalsIgnoreCase("sdhashfingerprintmatcher")) {

                currentScore = matcher.getScore(compare.getSdhash(), compareTo.getSdhash());

            } else {

                currentScore = matcher.getScore(compare.getFingerprint(), compareTo.getFingerprint());

            }

            //DEBUG
            if(update) {

                System.out.println("[] " + compare.getFileName() + "|" + compareTo.getFileName() + "|" + currentScore);

            }

            results.add(new Pair<>(compare.getFileName() + "|" + compareTo.getFileName(), currentScore));
        }

        return results;
    }
}