package edu.georgetown.gucs.tokenizers;


import java.io.*;
import java.util.Collections;
import java.util.List;
import java.util.Vector;
import java.util.zip.GZIPInputStream;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.Global;
import edu.georgetown.gucs.utility.Token;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

/**
 * Splits contents of a text file into tokens based on whitespace or by line. Used for {@linkplain java.util.Dictionary} creation.
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 */
public class FileTokenizer extends Tokenizer {

    ParseTokenizers parsedTokens_;
    protected String mode_ = "tokens";

    private List<String> tokenizers_;
    private String splitter = null;

    public FileTokenizer(String config) {
        this.tokenizers_ = new Vector<>();
        addTokenizers(config);
    }

    public FileTokenizer() {
        this.mode_ = "tokens";
    }

    //read file
    public void tokenizeFile(String fileName) {

        TokenizerList tokenizerList = new TokenizerList(this.tokenizers_);

        tokenizerList.setSplitter(this.splitter);
        this.tokenVectorMap = tokenizerList.tokenizeFile(fileName);

    }

    /*
        @Override
        public List<Token> tokenize(String fileName) {
            try {

                this.parsedTokens_ = new ParseTokenizers(fileName);
                List<Token> tokens = parsedTokens_.readFile();
                tokenVectorMap = parsedTokens_.getTokenVectorMap();

                return tokens;

            } catch (IOException e) {
                e.printStackTrace();
            }
            //TODO fix this code {looks ugly}
            return null;
        }

      /**
         * Splits the document into tokens
         */
    @Override
    public List<Token> tokenize(String filename) {
        return readFile(filename);
    }

    /**
     * Splits a text document into tokens
     *
     * @param filename the string filename of the document to split into tokens
     * @return the vector of tokens present in the file
     */
    protected List<Token> readFile(String filename) {
        BufferedReader buffer_reader = null;
        FileInputStream fis = null;
        GZIPInputStream gis = null;
        InputStreamReader isr = null;
        try {
            // TODO FileTokenizer: Assumes file is UTF-8 encoded (otherwise, it will spit out garbage)


            // Checks if file is gzip and creates appropriate buffered reader
            // buffer_reader = (filename.endsWith(".gz")) ? new BufferedReader(new InputStreamReader(new GZIPInputStream(
            //   new FileInputStream(filename)), "UTF-8")) : new BufferedReader(new InputStreamReader(new FileInputStream(
            //   filename), "UTF-8"));

            if (filename.endsWith(".gz")) {

                fis = new FileInputStream(filename);
                gis = new GZIPInputStream(fis);
                isr = new InputStreamReader(gis, Global.fileEncoding);
                buffer_reader = new BufferedReader(isr);
            } else {

                fis = new FileInputStream(filename);
                isr = new InputStreamReader(fis, Global.fileEncoding);
                buffer_reader = new BufferedReader(isr);

            }


            // buffer_reader = new BufferedReader(new InputStreamReader(new FileInputStream(filename), "UTF-8"));

            StringBuilder token = new StringBuilder("");
            int character;
            int byte_start = 0;
            int byte_end = 0;

            synchronized (this.tokenVectorMap) {
                this.tokenVectorMap.clear();
                List<Token> tokenVector = Collections.synchronizedList(new Vector<>());

                synchronized (tokenVector) {
                    while ((character = buffer_reader.read()) != -1) {
                        if (this.mode_.equalsIgnoreCase("lines")) {

                            if ('\n' == character || '\r' == character || '\f' == character) {

                                if (token.toString().trim().length() > 0) {

                                    tokenVector.add(new Token(token.toString().trim(), byte_start, byte_end));
                                    byte_start = byte_end;
                                }

                                byte_end += Character.charCount(character);
                                token.delete(0, token.length());

                            } else {
                                token.append((char) character);
                                byte_end += Character.charCount(character);
                            }

                        } else {
                            if (Character.isWhitespace(character)) {

                                if (token.toString().trim().length() > 0) {
                                    //  System.out.println(token.toString());
                                    tokenVector.add(new Token(token.toString().trim(),
                                            byte_start, byte_end));

                                    byte_start = byte_end;
                                }

                                byte_end += Character.charCount(character);
                                token.delete(0, token.length());

                            } else {
                                token.append((char) character);
                                byte_end += Character.charCount(character);
                            }
                        }
                    }
                    buffer_reader.close();

                    fis.close();
                    if (gis != null) {
                        gis.close();
                    }
                    isr.close();

                    // System.out.println("TESTING OUTPUT");
                    if (token.toString().trim().length() > 0) {

                        tokenVector.add(new Token(token.toString().trim(), byte_start, byte_end));
                    }
                    this.tokenVectorMap.put("null", tokenVector);

                    return tokenVector;
                }
            }
        } catch (IOException e) {
            ExceptionHandler.handleExceptionExit(e);
        } finally {
            try {

                if (buffer_reader != null) {
                    buffer_reader.close();
                }
                if (fis != null) {
                    fis.close();
                }
                if (gis != null) {
                    gis.close();
                }
                if (isr != null) {
                    isr.close();
                }
            } catch (IOException e) {
                ExceptionHandler.handleExceptionExit(e);
            }
        }
        return null;
    }

    /**
     * Adds tokenizers from a tokenizer configuration file
     *
     * @param config_file configuration file containing a list of tokenizers to use
     */
    @SuppressWarnings("unchecked")
    private void addTokenizers(String config_file) {
        try {
            File xml = new File(config_file);
            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build(xml);
            Element dict = doc.getRootElement();
            Element splitterElement;

            for (Element tokenizerElement : (List<Element>) dict.getChildren("tokenizer")) {
                String tokenizer = tokenizerElement.getAttributeValue("name");

                if (this.tokenizers_.size() == 0) {

                    if (!tokenizer.contains("FileTokenizer")) {
                        //("Dictionary: First tokenizer (" + tokenizer + ") must read from a file."));
                    } else if ((splitterElement = tokenizerElement.getChild("splitter")) != null) {
                        this.splitter = splitterElement.getAttributeValue("name") + " "
                                + splitterElement.getAttributeValue("value");
                    }
                }

                if (tokenizerElement.getAttributeValue("parameter") != null)
                    tokenizer += " " + (tokenizerElement.getAttributeValue("parameter"));
                this.tokenizers_.add(tokenizer.trim());
            }
        } catch (JDOMException | IOException e) {
            //  ExceptionHandler.handleExceptionExit(e);
        }
    }

    public void addTokenizers(List<String> tokenizerNames) {

        this.tokenizers_ = tokenizerNames;

    }


    public void setMode(String mode) {
        this.mode_ = mode.trim();
    }

}
