package edu.georgetown.gucs.utility;

import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Prints exception messages and stack traces before calling <code>System.exit(-1)</code>
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 */
public class ExceptionHandler {

    private static final Logger logger = Logger.getLogger(ExceptionHandler.class.getName());

    /**
     * Prints the message and stack trace for this Exception and exits; used with built in java exceptions (not
     * SDText-specific)
     *
     * @param exception the exception that provides information on error
     */
    public static void handleExceptionExit(Exception exception) throws RuntimeException {
        System.err.println("ExceptionHandler:" + exception.getMessage());
        System.err.println("ExceptionHandler " + Arrays.toString(exception.getStackTrace()));
        System.err.println("ExceptionHandler: " + exception.getMessage());
        logger.log(Level.SEVERE, Arrays.toString(exception.getStackTrace()));
        throw new RuntimeException(exception.getMessage());
        //System.exit(-1);
    }

    /**
     * Prints the message and stack trace for this Exception and throws a runtime exception; used with exceptions that are
     * SDText-specific (to give user feedback about error)
     *
     * @param exception the exception that provides information on error
     */
    public static void handleExceptionThrow(Exception exception) {
        logger.log(Level.SEVERE, exception.getMessage());
        throw new RuntimeException(exception.getMessage());
    }


}
