package sdtext;

import enums.CommandLineHelp;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**
 * @author Evan
 * @since 3/24/15.
 */
public class ApiTest extends CLIOptions {

    API api = new API();

    ApiTest(String[] args) {

        super(args);

    }

    /**
     *
     * @param fileName the file for sdtext to read
     * @return the contents of the file that sdText reads
     */
    public String readDoc(String fileName) {

        Path path = Paths.get(fileName);

        List<String> strings = new ArrayList<>();
        try {
            strings.addAll(Files.readAllLines(path.getFileName()));

        } catch (IOException e) {
            e.printStackTrace();
        }


        return listToString(strings);
    }

    /**
     *
     * @param strings converts a list of strings into a single string
     * @return a single string containing all of the lists contents
     */
    private String listToString(List<String> strings) {

        StringBuilder docContent = new StringBuilder();

        strings.forEach(docContent::append);

        return docContent.toString();
    }

    /**
     * Reads a file that contains a list of fingerprint file names in order to compare them with text
     * @param fileName the filename which contains a list of fingerprint files to compare to text
     * @return the fingerprint files that are named in filename
     */
    private Vector<String> readFileList(String fileName) {


        Path path = Paths.get(fileName);

        Vector<String> strings = new Vector<>();
        try {
            strings.addAll(Files.readAllLines(path.getFileName()));

        } catch (IOException e) {
            System.out.println("no files");
            e.printStackTrace();
        }

        return strings;
    }


    /**
     * calls CLIOptions in addition to allowing API access in order to conduct
     * command line testing
     */
    public void runTest() {

            runProgram();


        switch (this.programName_) {

            //java -jar build/sdtext.jar Fingerprintstotext -i 4 -f fing.directory -m CosineSimilarityFingerprintMatcher -s 0
            case "FINGERPRINTSTOTEXT": {

                //input = document to fingerprint
                isNull(CommandLineHelp.FINGERPRINTSTOTEXT, input, fingerprint);

                matcher.value = (matcher.value == null) ? "CosineSimilarityFingerprintMatcher" : matcher.value;
                minScore.value = (minScore.value > 0) ? minScore.value : 0;

                String contents = readDoc(input.value);
                Vector<String> fingerprintFiles = readFileList(fingerprint.value);

                api.cmpFingerprintsToText((int) minScore.value,
                        matcher.value,
                        null,
                        contents,
                        fingerprintFiles);

                break;
            }
            //java -jar build/sdtext.jar CompareFingToDirectory -m CosineSimilarityFingerprintMatcher -s 0 -f fed4.fing -p ../sdtext/doc/input_files/federalist/
            case "COMPAREFINGTODIRECTORY": {

                isNull(CommandLineHelp.COMPAREFINGTODIR, directory, fingerprint);

                matcher.value = (matcher.value == null) ? "CosineSimilarityFingerprintMatcher" : matcher.value;
                minScore.value = (minScore.value > 0) ? minScore.value : 0;

                api.cmpFingerprintToDirectory((int) minScore.value, matcher.value, fingerprint.value, directory.value);

                break;
            }
            // java -jar build/sdtext.jar CreateDictTrim -l 0 -h .6 -c ../configuration_files/tokenizers_config.xml -p ../enron -d enronV2.dict
            case "CREATEDICTTRIM": {
                isNull(CommandLineHelp.CREATEDICTTRIM, directory, fingerprint, minIDF, maxIDF);

                //dictionary is the output dictionary filename
                api.createDictAndTrim(minIDF.value, maxIDF.value, config.value, directory.value, dictionary.value);

                break;
            }
            //java -jar build/sdtext.jar BitVectorFingerprinter -i ../enron_9 -d enronV2.dict -o enronV2_9.fing -c ../configuration_files/fingerprinter_config.xml//
            case "BITVECTORFINGERPRINTER": {

                api.createFingerprint(fingerprint.value, dictionary.value, output.value);
                break;
            }

            default: {
                printHelp();
            }
        }
    }


}
